//  hwdeint.h
//
//  hwdeint main
//
//	Copyright (c) 2005 N099
//
#ifndef _HWDEINT_H_
#define _HWDEINT_H_

#include <vector>
#include <string>

#include "transip2.h"
#include "hwdeintdefs.h"
#include "tray.h"


// interface
interface IHWDeint : public IUnknown
{
	STDMETHOD(is_tray_visible)(void) const = 0;
	STDMETHOD(hide_tray)(void) = 0;
	STDMETHOD(show_tray)(void) = 0;
	STDMETHOD(get_cfg)(CONFIG *) = 0;
	STDMETHOD(set_cfg)(CONFIG) = 0;
	STDMETHOD(get_cfg_am2)(CFG_AM2 *) = 0;
	STDMETHOD(set_cfg_am2)(CFG_AM2) = 0;
	STDMETHOD(get_cfg_vif2)(CFG_VIF2 *) = 0;
	STDMETHOD(get_cfg_ar)(CFG_AR *) = 0;
	STDMETHOD(set_cfg_ar)(CFG_AR) = 0;
	STDMETHOD(get_cfg_color)(int *) = 0;
	STDMETHOD(set_cfg_color)(int) = 0;
	STDMETHOD(get_dsflist)(std::vector<std::wstring> *) = 0;
	STDMETHOD(set_dsflist)(const std::vector<std::wstring> *) = 0;
};


// main class
class CHWDeintFilter : public CTransInPlaceFilter2, public IHWDeint, public ISpecifyPropertyPages
{
private:
	CCritSec m_csShared;
	TrayIcon *m_pTi;
	CONFIG   m_conf;
	int      m_ARflag;

	std::vector<std::wstring> m_dsflist;

	HRESULT SearchHWDeint(PIN_DIRECTION dir, IPin *pPin);
	HRESULT CheckFilter(IPin *pPin);
	HRESULT CheckFilter(PIN_DIRECTION dir, IPin *pPin);
	HRESULT AddVIF2Flag(CMediaType *pMediaType);
	HRESULT AddVIF2Flag(AM_MEDIA_TYPE *pMediaType);
	HRESULT AddVIF2Flag(VIDEOINFOHEADER2 *pVih);
public:

    static CUnknown *WINAPI CreateInstance(LPUNKNOWN punk, HRESULT *phr);

	STDMETHODIMP NonDelegatingQueryInterface(REFIID riid, void ** ppv);
	DECLARE_IUNKNOWN;

	CHWDeintFilter(LPUNKNOWN punk, HRESULT *phr);
	~CHWDeintFilter();

	HRESULT CheckInputType(const CMediaType * mtIn);
	HRESULT GetMediaType(int iPosition, CMediaType *pMediaType);
	HRESULT Transform(IMediaSample *pDest);
	HRESULT CompleteConnect(PIN_DIRECTION dir,IPin *pReceivePin);

	STDMETHODIMP GetPages(CAUUID * pPages);
	STDMETHODIMP FreePages(CAUUID * pPages);

	// tray
	STDMETHODIMP is_tray_visible(void) const {
		return m_conf.misc.tray;
	}
	STDMETHODIMP hide_tray(void){
		CAutoLock lock(&m_csShared);
		m_pTi->hide();
		return S_OK;
	}
	STDMETHODIMP show_tray(void){
		CAutoLock lock(&m_csShared);
		m_pTi->show();
		return S_OK;
	}

	//IHWDeint interfaces
	STDMETHODIMP get_cfg(CONFIG *);
	STDMETHODIMP set_cfg(CONFIG);
	STDMETHODIMP get_cfg_am2(CFG_AM2 *);
	STDMETHODIMP set_cfg_am2(CFG_AM2);
	STDMETHODIMP get_cfg_vif2(CFG_VIF2 *);
	STDMETHODIMP get_cfg_ar(CFG_AR *);
	STDMETHODIMP set_cfg_ar(CFG_AR);
	STDMETHODIMP get_cfg_color(int *);
	STDMETHODIMP set_cfg_color(int);
	STDMETHODIMP get_dsflist(std::vector<std::wstring> *);
	STDMETHODIMP set_dsflist(const std::vector<std::wstring> *);

};

#endif  /* _HWDEINT_H_ */
