//------------------------------------------------------------------------------
// File: TransIP2.h
//
// Desc: modified transinplace filter
//
// Copyright (c) Microsoft Corporation.  All rights reserved.
// Copyright (c) 2005 N099
//------------------------------------------------------------------------------

#ifndef __TRANSIP2__
#define __TRANSIP2__

// ==================================================
// Implements the allocator
// ==================================================
class CPassThruAllocator : public CUnknown,// A non delegating IUnknown
                       public IMemAllocator, // The interface we support
                       public CCritSec             // Provides object locking
{
private:
	IMemAllocator *m_pAllocator;
	BOOL m_convert;
public:
    CPassThruAllocator(
        TCHAR *, LPUNKNOWN, HRESULT *,
        IMemAllocator * pAllocator, BOOL convert = FALSE);
#ifdef UNICODE
    CPassThruAllocator(
        CHAR *, LPUNKNOWN, HRESULT *,
        IMemAllocator * pAllocator, BOOL convert = FALSE);
#endif
    ~CPassThruAllocator();

	void SetConvert(BOOL cnv)
	{ m_convert = cnv; }

	IMemAllocator * PeekAllocator() const
	{ return m_pAllocator; }

    DECLARE_IUNKNOWN

    STDMETHODIMP NonDelegatingQueryInterface(REFIID riid, void **ppv);

	//IMemAllocator interface
    STDMETHODIMP SetProperties(
		    ALLOCATOR_PROPERTIES* pRequest,
		    ALLOCATOR_PROPERTIES* pActual);
    STDMETHODIMP GetProperties(
		    ALLOCATOR_PROPERTIES* pProps);
    STDMETHODIMP Commit();
    STDMETHODIMP Decommit();
    STDMETHODIMP GetBuffer(IMediaSample **ppBuffer,
                           REFERENCE_TIME * pStartTime,
                           REFERENCE_TIME * pEndTime,
                           DWORD dwFlags);
    STDMETHODIMP ReleaseBuffer(IMediaSample *pBuffer);
};

// ======================================================================
// This is the com object that represents a simple transform filter. It
// supports IBaseFilter, IMediaFilter and two pins through nested interfaces
// ======================================================================
class CTransInPlaceFilter2;

// ==================================================
// Implements the input pin
// ==================================================

class CTransInPlaceInputPin2 : public CTransformInputPin
{

protected:
    CTransInPlaceFilter2 * const m_pTIPFilter;    // our filter
    BOOL                 m_bReadOnly;    // incoming stream is read only
	BOOL                 m_UsingPassThruAllocator; // using custom alloc

public:

    CTransInPlaceInputPin2(
        TCHAR                *pObjectName,
        CTransInPlaceFilter2 *pFilter,
        HRESULT              *phr,
        LPCWSTR              pName);

    // --- IMemInputPin -----

    // Say whether media type is acceptable.
    HRESULT CheckMediaType(const CMediaType* pmt);

    // Return our upstream allocator
    STDMETHODIMP GetAllocator(IMemAllocator ** ppAllocator);

    // get told which allocator the upstream output pin is actually
    // going to use.
    STDMETHODIMP NotifyAllocator(IMemAllocator * pAllocator,
                                 BOOL bReadOnly);

    // Allow the filter to see what allocator we have
    // N.B. This does NOT AddRef
    IMemAllocator * PeekAllocator(void);

    // Pass this on downstream if it ever gets called.
    STDMETHODIMP GetAllocatorRequirements(ALLOCATOR_PROPERTIES *pProps);

    HRESULT CompleteConnect(IPin *pReceivePin);

    inline const BOOL ReadOnly() { return m_bReadOnly ; }

	// set convert
	HRESULT SetConvert(BOOL cnv);

};  // CTransInPlaceInputPin2

// ==================================================
// Implements the output pin
// ==================================================

class CTransInPlaceOutputPin2 : public CTransformOutputPin
{

protected:
    // m_pFilter points to our CBaseFilter
    CTransInPlaceFilter2 * const m_pTIPFilter;

public:

    CTransInPlaceOutputPin2(
        TCHAR               *pObjectName,
        CTransInPlaceFilter2 *pFilter,
        HRESULT             *phr,
        LPCWSTR              pName);


    // --- CBaseOutputPin ------------

    // Say whether media type is acceptable.
    HRESULT CheckMediaType(const CMediaType* pmt);

    //  This just saves the allocator being used on the output pin
    //  Also called by input pin's GetAllocator()
    void SetAllocator(IMemAllocator * pAllocator);

    IMemInputPin * ConnectedIMemInputPin()
        { return m_pInputPin; }

    // Allow the filter to see what allocator we have
    // N.B. This does NOT AddRef
    IMemAllocator * PeekAllocator() const
        {  return m_pAllocator; }

    HRESULT CompleteConnect(IPin *pReceivePin);

};  // CTransInPlaceOutputPin2


class AM_NOVTABLE CTransInPlaceFilter2 : public CTransformFilter
{

public:

    // map getpin/getpincount for base enum of pins to owner
    // override this to return more specialised pin objects

    virtual CBasePin *GetPin(int n);

public:

    //  Set bModifiesData == false if your derived filter does
    //  not modify the data samples (for instance it's just copying
    //  them somewhere else or looking at the timestamps).

    CTransInPlaceFilter2(TCHAR *, LPUNKNOWN, REFCLSID clsid, HRESULT *,
                        bool bModifiesData = true,
                        bool convert = true);
#ifdef UNICODE
    CTransInPlaceFilter2(CHAR *, LPUNKNOWN, REFCLSID clsid, HRESULT *,
                        bool bModifiesData = true,
                        bool convert = true);
#endif
    // The following are defined to avoid undefined pure virtuals.
    // Even if they are never called, they will give linkage warnings/errors

    // This is called when we actually have to provide out own allocator.
    HRESULT DecideBufferSize(IMemAllocator*, ALLOCATOR_PROPERTIES *);

    // The functions which call this in CTransform are overridden in this
    // class to call CheckInputType with the assumption that the type
    // does not change.  In Debug builds some calls will be made and
    // we just ensure that they do not assert.
    HRESULT CheckTransform(const CMediaType *mtIn, const CMediaType *mtOut)
    {
        return S_OK;
    };


    // =================================================================
    // ----- You may want to override this -----------------------------
    // =================================================================

    virtual HRESULT CompleteConnect(PIN_DIRECTION dir,IPin *pReceivePin) PURE;

    // chance to customize the transform process
    virtual HRESULT Receive(IMediaSample *pSample);

    // =================================================================
    // ----- You MUST override these -----------------------------------
    // =================================================================

    virtual HRESULT Transform(IMediaSample *pSample) PURE;

protected:

    IMediaSample * CTransInPlaceFilter2::Copy(IMediaSample *pSource);

    bool  m_bModifiesData; // Does this filter change the data?
	bool  m_convert; // convert mediatype to the upstream

    // these hold our input and output pins

    friend class CTransInPlaceInputPin2;
    friend class CTransInPlaceOutputPin2;

    CTransInPlaceInputPin2  *InputPin() const
    {
        return (CTransInPlaceInputPin2 *)m_pInput;
    };
    CTransInPlaceOutputPin2 *OutputPin() const
    {
        return (CTransInPlaceOutputPin2 *)m_pOutput;
    };

    //  Are the input and output allocators different?
    BOOL UsingDifferentAllocators() const
    {
        return InputPin()->PeekAllocator() != OutputPin()->PeekAllocator();
    }
}; // CTransInPlaceFilter2

//util
STDAPI ConvertVideoInfo2ToVideoInfo(AM_MEDIA_TYPE *);

#endif /* __TRANSIP2__ */

